package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

public class AddressBook extends AbstractBehavior<AddressBook.Message> {
    private final ArrayList<ActorRef<Customer.Message>> customers;

    public interface Message {}

    // Nachricht, wenn ein Empfänger nach einer Adresse fragt
    public static final class CustomerMessage implements Message {
        public final ActorRef<Customer.Message> fragender;
        public final String nameFragender;
        public final ActorRef<DeliveryCar.Message> truckReference;

        public CustomerMessage(ActorRef<Customer.Message> fragender, String nameFragender, ActorRef<DeliveryCar.Message> truckReference) {
            this.fragender = fragender;
            this.nameFragender = nameFragender;
            this.truckReference = truckReference;
        }
    }

    // Nachricht, um die Liste der Kunden abzurufen
    public static final class GetCustomers implements Message {
        public final ActorRef<List<ActorRef<Customer.Message>>> replyTo;

        public GetCustomers(ActorRef<List<ActorRef<Customer.Message>>> replyTo) {
            this.replyTo = replyTo;
        }
    }

    private AddressBook(ActorContext<Message> context, ArrayList<ActorRef<Customer.Message>> customers) {
        super(context);
        this.customers = new ArrayList<>(customers);
        for (ActorRef<Customer.Message> cst : this.customers) {
            cst.tell(new Customer.AddressHello(getContext().getSelf()));
        }
    }

    public static Behavior<Message> create(ArrayList<ActorRef<Customer.Message>> customers) {
        return Behaviors.setup(context -> new AddressBook(context, customers));
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(CustomerMessage.class, this::onCustomerMessage)
                .onMessage(GetCustomers.class, this::onGetCustomers)
                .build();
    }

    // Methode, um einen zufälligen Empfänger auszugeben
    private Behavior<Message> onCustomerMessage(CustomerMessage msg) {
        Random random = new Random();
        int index = random.nextInt(customers.size());
        ActorRef<Customer.Message> addressOf = customers.get(index);
        getContext().getLog().info("{} asked for an address CAR: {} ", msg.nameFragender, msg.truckReference.path().name());
        msg.fragender.tell(new Customer.AddressMessage(addressOf, msg.truckReference));
        return this;
    }

    // Methode, um die Liste der Kunden zu senden
    private Behavior<Message> onGetCustomers(GetCustomers msg) {
        msg.replyTo.tell(Collections.unmodifiableList(customers));
        return this;
    }
}
