package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.util.ArrayList;
import java.util.List;

import static com.example.AkkaMainSystem.getDistributionCenter;

public class DeliveryCar extends AbstractBehavior<DeliveryCar.Message> {

    private final ArrayList<Paket> pakete = new ArrayList<>();
    private final ArrayList<ActorRef<Customer.Message>> route;
    private final int maxCapacity = 3;
    private int globalIndex = 0;

    public interface Message {}

    public record LoadMessage(List<Paket> pakete) implements Message {}

    public record PickupResponse(Paket paket) implements Message {}

    public record StartRoute() implements Message {}

    private DeliveryCar(ActorContext<Message> context, ArrayList<ActorRef<Customer.Message>> route) {
        super(context);
        this.route = route;
    }

    public static Behavior<Message> create(List<ActorRef<Customer.Message>> route) {
        return Behaviors.setup(context -> new DeliveryCar(context, new ArrayList<>(route)));
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(PickupResponse.class, this::onPickupResponse)
                .onMessage(LoadMessage.class, this::onLoadMessage)
                .onMessage(StartRoute.class, this::onStartRoute)
                .build();
    }

    private Behavior<Message> onLoadMessage(LoadMessage msg) {
        pakete.addAll(msg.pakete);
        // Nach dem Laden der Pakete zum nächsten Kunden fahren
        sendPickupMessage();
        return this;
    }

    private Behavior<Message> onPickupResponse(PickupResponse rsp) {
        if (rsp.paket == null || rsp.paket.inhalt == null) {
            globalIndex++;
            checkNextStop();
        } else {
            if (pakete.size() < maxCapacity) {
                pakete.add(rsp.paket);
                getContext().getLog().info("Aktuelle Anzahl der Pakete im Truck: {}", pakete.size());
            }
            globalIndex++;
            checkNextStop();
        }
        return this;
    }
    private void checkNextStop() {
        getContext().getLog().info("globalIndex {}", globalIndex);

        if (globalIndex >= route.size()) {
            getContext().getLog().info("Wagen kehrt zum Verteilzentrum zurück.");
            DistributionCenter.ArriveMessage antwort = new DistributionCenter.ArriveMessage(pakete, getContext().getSelf());
            getDistributionCenter().tell(antwort);
            // Setze globalIndex auf 0, damit der Wagen weitermachen kann
            globalIndex = 0;
            return;
        } else {
            getContext().getLog().info("ich bin bei {}", route.get(globalIndex).path().name());
            if (pakete.size() >= maxCapacity) {
                deliverPackages();
            } else {
                sendPickupMessage();
            }
        }
    }

    private Behavior<Message> onStartRoute(StartRoute msg) {
        globalIndex = 0;
        checkNextStop();
        return this;
    }

    private void sendPickupMessage() {
        if (globalIndex < route.size()) {
            ActorRef<Customer.Message> nextCustomer = route.get(globalIndex);
            nextCustomer.tell(new Customer.PickUpMessage(getContext().getSelf(), "Wagen kann Paket aufnehmen"));
        } else {
            getContext().getLog().info("Keine weiteren Kunden in der Route.");
        }
    }

    private void deliverPackages() {
        if (globalIndex < route.size()) {
            ActorRef<Customer.Message> nextCustomer = route.get(globalIndex);
            for (int i = 0; i < pakete.size(); i++) {
                Paket paket = pakete.get(i);
                if (paket.empfaenger == nextCustomer) {
                    nextCustomer.tell(new Customer.DeliveryMessage(paket));
                    pakete.remove(i);
                    i--; // Index anpassen, da ein Element entfernt wurde
                }
            }
        }
    }
}
