package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class DistributionCenter extends AbstractBehavior<DistributionCenter.Message> {

    private final ActorRef<AddressBook.Message> addressBook;
    public static class StartAllRoutes implements Message {}

    private final List<Paket> lagerraum = new ArrayList<>();
    private final List<ActorRef<DeliveryCar.Message>> deliveryCars = new ArrayList<>();


    private DistributionCenter(ActorContext<Message> context, ActorRef<AddressBook.Message> addressBook) {
        super(context);
        this.addressBook = addressBook;

        // Erhalten Sie die replyTo-Referenz aus dem Kontext
        ActorRef<List<ActorRef<Customer.Message>>> replyTo = context.spawnAnonymous(Behaviors.<List<ActorRef<Customer.Message>>>receive((ctx, message) -> {
            if (message instanceof List) {
                @SuppressWarnings("unchecked")
                List<ActorRef<Customer.Message>> customers = (List<ActorRef<Customer.Message>>) message;
                ctx.getLog().info("Kundenliste erhalten: {}", customers);
                // Nachricht senden, um Routen zu generieren
                context.getSelf().tell(new GenerateRoutes(customers));
                return Behaviors.same();
            }
            return Behaviors.unhandled();
        }));

        // Anfrage an das AddressBook senden, um die Kundenliste abzurufen
        addressBook.tell(new AddressBook.GetCustomers(replyTo));
    }

    public static Behavior<Message> create(ActorRef<AddressBook.Message> addressBook) {
        return Behaviors.setup(context -> new DistributionCenter(context, addressBook));
    }

    interface Message {}

    public static class ArriveMessage implements Message {
        private final List<Paket> pakete;
        private final ActorRef<DeliveryCar.Message> truck;

        public ArriveMessage(List<Paket> pakete, ActorRef<DeliveryCar.Message> truck) {
            this.pakete = pakete;
            this.truck = truck;
        }

        public List<Paket> getPakete() {
            return pakete;
        }

        public ActorRef<DeliveryCar.Message> getTruck() {
            return truck;
        }
    }



    public static class GenerateRoutes implements Message {
        private final List<ActorRef<Customer.Message>> customers;

        public GenerateRoutes(List<ActorRef<Customer.Message>> customers) {
            this.customers = customers;
        }

        public List<ActorRef<Customer.Message>> getCustomers() {
            return customers;
        }
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(ArriveMessage.class, this::onArriveMessage)
                .onMessage(GenerateRoutes.class, this::onGenerateRoutes)
                .onMessage(StartAllRoutes.class, this::onStartAllRoutes)
                .build();
    }

    private Behavior<Message> onArriveMessage(ArriveMessage msg) {
        // Füge alle Pakete aus der Ankunftsnachricht dem Lagerraum hinzu
        List<Paket> arrivedPackages = msg.getPakete();
        lagerraum.addAll(arrivedPackages);
        String arrivedPackagesString = "";
        for (int i = 0; i < arrivedPackages.size(); i++) {
            arrivedPackagesString += arrivedPackages.get(i).inhalt;
            if (i < arrivedPackages.size() - 1) {
                arrivedPackagesString += ", "; // Fügen Sie ein Trennzeichen hinzu, außer für das letzte Paket
            }
        }
        getContext().getLog().info("Angekommene Pakete: {} Anz Packeten in Lager: {}", arrivedPackagesString, lagerraum.size());

        // Entferne zufällig 3 Pakete aus dem Lagerraum und sende sie dem angekommenen Wagen
        int numPackagesToSend = Math.min(3, lagerraum.size());
        List<Paket> packagesToSend = new ArrayList<>();
        for (int i = 0; i < numPackagesToSend; i++) {
            packagesToSend.add(lagerraum.remove(0));
        }

        msg.getTruck().tell(new DeliveryCar.LoadMessage(packagesToSend));
        msg.getTruck().tell(new DeliveryCar.StartRoute()); // Starte die Route
        // Ausgabe, wie viele Pakete nun im Verteilzentrum vorhanden sind
        getContext().getLog().info("Pakete im Verteilzentrum nach Laden: {}", lagerraum.size());
        return this;
    }



    private Behavior<Message> onGenerateRoutes(GenerateRoutes msg) {
        List<ActorRef<Customer.Message>> customers = msg.getCustomers();
        generateRoutes(customers);
        return this;
    }

    private void generateRoutes(List<ActorRef<Customer.Message>> customers) {
        // Überprüfen, ob die Kundenliste null oder leer ist
        if (customers == null || customers.isEmpty()) {
            getContext().getLog().info("Kundenliste ist leer oder null. Keine Routen generiert.");
            return;
        }

        // Erstelle vier Lastwagen mit zufälligen Routen
        for (int i = 0; i < 2; i++) {
            /*List<ActorRef<Customer.Message>> route = generateRandomRoute(customers);
            getContext().spawn(DeliveryCar.create(route), "deliveryCar" + i+1);
            getContext().getLog().info("Lieferwagen {} erstellt mit Route: {}", i+1, route.toString());*/
            List<ActorRef<Customer.Message>> route = generateRandomRoute(customers);
            ActorRef<DeliveryCar.Message> deliveryCar = getContext().spawn(DeliveryCar.create(route), "deliveryCar" + (i + 1));

            String RouteStr = "";
            for (int j = 0; j < route.size(); j++) {
                RouteStr += route.get(j).path().name();
                if (j < route.size() - 1) {
                    RouteStr += ", "; // Fügen Sie ein Trennzeichen hinzu, außer für das letzte Paket
                }
            }
            getContext().getLog().info("Lieferwagen {} erstellt mit Route: {}", i + 1, RouteStr);
            deliveryCar.tell(new DeliveryCar.StartRoute());

        }
    }
    private Behavior<Message> onStartAllRoutes(StartAllRoutes msg) {
        for (ActorRef<DeliveryCar.Message> deliveryCar : deliveryCars) {
            deliveryCar.tell(new DeliveryCar.StartRoute());
        }
        return this;
    }

    // Dummy-Methode zum Generieren einer zufälligen Route
    private List<ActorRef<Customer.Message>> generateRandomRoute(List<ActorRef<Customer.Message>> customers) {
        List<ActorRef<Customer.Message>> route = new ArrayList<>(customers);
        Collections.shuffle(route);
        return route;
    }
}